﻿using System;
using System.IO;
using System.Text;
using System.Threading;
using System.Diagnostics;
using System.Globalization;

namespace Logging
{
    #region Enums

    public enum SwitchReasons
    {
        None = 0,
        Size = 1,
        Day = 2,
        Week = 3,
        Month = 4
    }

    public enum OutputFormat
    {
        Default = 0,
        Fixed = 1,
        CSV = 2,
        ExcelCSV = 3
    }

    #endregion

    public class BlogTraceListener : TraceListener
    {
        #region Private Konstanten

        private const string defaultInitDataPath = @"C:\Temp\";
        private const string defaultInitDataFileName = "LogFile-{0}-{1}-{2:D8}-{3:D8}";
        private const string defaultInitDataFileExtention = "log";
        private const string defaultInitDataSwitchReason = "T";
        private const string defaultInitDataOutputFormat = "FIX";

        private const string defaultInitData =
            defaultInitDataPath +
            defaultInitDataFileName + "." +
            defaultInitDataFileExtention + ";" +
            defaultInitDataSwitchReason + "," +
            defaultInitDataOutputFormat;

        private const int maxCategoryNameLength = 12;
        private readonly string messageFormatStringFixedFirstLine = string.Format("{{0,-{0}:C}} {{1:0000}} {{2:00000}} {{3:00000}} {{4}} {{5}}", maxCategoryNameLength);
        private readonly string messageFormatStringFixedLines = string.Format(string.Format("{{0,-{0}:C}}", maxCategoryNameLength + 38), " ") + "{0}";
        private readonly string messageFormatStringCSV = "{0},{1},{2},{3},{4},{5}";

        private const string defaultCategory = "NONE";
        private const string defaultCategoryTrace = "TRACE";

        private const string parameterSwitchReasonDay = "D";
        private const string parameterSwitchReasonTag = "T";
        private const string parameterSwitchReasonWeek = "W";
        private const string parameterSwitchReasonMonth = "M";

        private const string parameterOutputFormatFixed = "FIX";
        private const string parameterOutputFormatCSV = "CSV";
        private const string parameterOutputFormatExcelCSV = "EXCSV";

        #endregion

        #region Private Variablen

        private SwitchReasons switchReason = SwitchReasons.None;
        private OutputFormat outputFormat = OutputFormat.Default;

        private bool logFileIsOpen = false;
        private string logFileName = null;
        private string logFileExtention = null;
        private long logFileMaxLength = 0;
        private FileInfo logFileInfo = null;
        private StreamWriter logFileStream = null;
        private DateTime logFileDateTime;

        private long loggedMessages = 0;

        #endregion

        #region Konstruktor(en)

        /// <summary>
        /// Constructor
        /// </summary>
        public BlogTraceListener()
            : this(defaultInitData)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="initializeData">Initialisierungsparameter</param>
        public BlogTraceListener(string initializeData)
        {
            initData(initializeData);
        }

        #endregion

        #region Destuktor

        /// <summary>
        /// Destructor 
        /// </summary>
        ~BlogTraceListener()
        {
            WriteLine("Trace beendet", defaultCategoryTrace);
        }

        #endregion

        #region Private Methoden

        /// <summary>
        /// Initialisierung der Variablen, Schreiben der Header-Informationen
        /// </summary>
        /// <param name="initializeData">Initialisierungs-String (z. B. aus app.config)</param>
        private void initData(string initializeData)
        {
            if (initializeData == null || initializeData.Length == 0)
                initializeData = defaultInitData;

            string[] tempArray;
            char[] delimiterArray = { ';', ',' };
            StringBuilder tempFilename;

            Process currentProcess = Process.GetCurrentProcess();

            tempArray = initializeData.Split(delimiterArray, 3);

            if (tempArray.Length > 0)
                tempFilename = new StringBuilder();
            else
                tempFilename = new StringBuilder(defaultInitDataPath);

            tempFilename.AppendFormat(tempArray[0],
                System.Environment.MachineName,
                System.Environment.UserName,
                currentProcess.Id,
                currentProcess.SessionId);
            logFileName = tempFilename.ToString();

            logFileName = Environment.ExpandEnvironmentVariables(logFileName);
            logFileExtention = Path.GetExtension(logFileName);

            logFileName = logFileName.Replace(logFileExtention, "");
            logFileExtention = logFileExtention.Replace(".", "");

            if (logFileExtention == "")
                logFileExtention = defaultInitDataFileExtention;

            logFileMaxLength = 0;
            if (tempArray.Length > 1)
            {
                switch (tempArray[1].Trim().ToUpper())
                {
                    case parameterSwitchReasonDay:
                    case parameterSwitchReasonTag:
                        switchReason = SwitchReasons.Day;
                        break;
                    case parameterSwitchReasonWeek:
                        switchReason = SwitchReasons.Week;
                        break;
                    case parameterSwitchReasonMonth:
                        switchReason = SwitchReasons.Month;
                        break;
                    default:
                        switchReason = SwitchReasons.None;
                        if (Int64.TryParse(tempArray[1], out logFileMaxLength))
                            if (logFileMaxLength > 0)
                                switchReason = SwitchReasons.Size;
                        break;
                }
            }
            else
                switchReason = SwitchReasons.None;

            if (tempArray.Length > 2)
            {
                switch (tempArray[2].Trim().ToUpper())
                {
                    case parameterOutputFormatFixed:
                        outputFormat = OutputFormat.Fixed;
                        break;
                    case parameterOutputFormatCSV:
                        outputFormat = OutputFormat.CSV;
                        break;
                    case parameterOutputFormatExcelCSV:
                        outputFormat = OutputFormat.ExcelCSV;
                        break;
                    default:
                        outputFormat = OutputFormat.Default;
                        break;
                }
            }
            else
                outputFormat = OutputFormat.Default;

            logFileInfo = new FileInfo(logFileName + "." + logFileExtention);

            if (logFileInfo.Exists)
                logFileDateTime = logFileInfo.CreationTime;
            else
                logFileDateTime = DateTime.Now;

            fileOpen();

#if DEBUG
            WriteLine("Trace gestartet - DebugVersion", defaultCategoryTrace);
#else
            WriteLine("Trace gestartet - ReleaseVersion", defaultCategoryTrace);
#endif
            WriteLine("initializeData=\"" + initializeData + "\"", defaultCategoryTrace);

            string parameterSwitchReason = "";
            switch (switchReason)
            {
                case SwitchReasons.None:
                    parameterSwitchReason = "Kein Dateiwechsel";
                    break;
                case SwitchReasons.Size:
                    parameterSwitchReason = string.Format("Dateiwechsel bei Dateigröße von {0} bytes", logFileMaxLength);
                    break;
                case SwitchReasons.Day:
                    parameterSwitchReason = "Dateiwechsel täglich";
                    break;
                case SwitchReasons.Week:
                    parameterSwitchReason = "Dateiwechsel wöchentlich";
                    break;
                case SwitchReasons.Month:
                    parameterSwitchReason = "Dateiwechsel monatlich";
                    break;
            }
            WriteLine(parameterSwitchReason, defaultCategoryTrace);

            writeHeader();

            fileClose();
        }

        /// <summary>
        /// Öffnet die Ausgabedatei, sofern diese noch nicht geöffnet ist
        /// </summary>
        private void fileOpen()
        {
            if (!logFileIsOpen)
            {
                logFileStream = logFileInfo.AppendText();
                logFileStream.AutoFlush = true;
                logFileIsOpen = true;
            }
        }

        /// <summary>
        /// Schließt die Ausgabedatei, sofern diese geöffnet ist
        /// </summary>
        private void fileClose()
        {
            if (logFileIsOpen)
            {
                logFileStream.Close();
                logFileIsOpen = false;
            }
        }

        /// <summary>
        /// Verschiebt die Ausgabedatei beim Dateiwechsel
        /// </summary>
        /// <param name="dateTime">Zeitstempel für den Dateiwechsel</param>
        /// <returns>Dateiname der alten Ausgabedatei</returns>
        private string fileMove(DateTime dateTime)
        {
            StringBuilder moveFileName;

            moveFileName = new StringBuilder();
            moveFileName.Append(logFileName);
            moveFileName.AppendFormat("-{0}.{1}",
                dateTime.ToString("yyyy-MM-dd-HH-mm"),
                logFileExtention);

            logFileDateTime = dateTime;

            fileClose();

            logFileInfo.MoveTo(moveFileName.ToString());

            return moveFileName.ToString();
        }

        /// <summary>
        /// Testet, ob eine der Bedingungen für Ausgabedateiwechsel erfüllt ist 
        /// und führt  ggf. den Dateiwechsel aus
        /// </summary>
        private void checkSwitchReason()
        {
            bool switchFlag;
            string reason = null;
            DateTime dateTime = DateTime.Now;

            logFileInfo.Refresh();

            switchFlag = false;
            switch (switchReason)
            {
                case SwitchReasons.Size:
                    if (logFileInfo.Length >= logFileMaxLength && logFileMaxLength != 0)
                    {
                        switchFlag = true;
                        reason = "Dateiwechsel: Dateilänge überschritten";
                    }
                    break;

                case SwitchReasons.Day:
                    if (logFileDateTime.DayOfYear != dateTime.DayOfYear)
                    {
                        switchFlag = true;
                        reason = "Dateiwechsel: Tagesgrenze überschritten";
                    }
                    break;

                case SwitchReasons.Week:
                    CultureInfo culture = System.Threading.Thread.CurrentThread.CurrentCulture;
                    Calendar cal = culture.Calendar;
                    int weekFile = cal.GetWeekOfYear(logFileDateTime, culture.DateTimeFormat.CalendarWeekRule, culture.DateTimeFormat.FirstDayOfWeek);
                    int week = cal.GetWeekOfYear(dateTime, culture.DateTimeFormat.CalendarWeekRule, culture.DateTimeFormat.FirstDayOfWeek);

                    if (weekFile != week)
                    {
                        switchFlag = true;
                        reason = "Dateiwechsel: Wochengrenze überschritten";
                    }
                    break;

                case SwitchReasons.Month:
                    if (logFileDateTime.Month != dateTime.Month)
                    {
                        switchFlag = true;
                        reason = "Dateiwechsel: Monatsgrenze überschritten";
                    }
                    break;
                default:
                    break;
            }

            if (switchFlag)
            {
                writeSwitchReason(reason);

                string oldFileName = fileMove(dateTime);
                logFileInfo = null;
                logFileInfo = new FileInfo(logFileName + "." + logFileExtention);

                fileOpen();

                writeSwitchReason(reason);
                WriteLine("Dateiwechsel, alte Datei unter " + oldFileName.ToString(), defaultCategoryTrace);
                writeHeader();
            }
        }

        /// <summary>
        /// Schreibt Basisinformationen in die Ausgabedatei
        /// </summary>
        public void writeHeader()
        {
            bool isOpen = logFileIsOpen;
            fileOpen();

            Process process = Process.GetCurrentProcess();

            WriteLine("Assembly:  " + this.GetType().Assembly.GetName().Version.ToString(), defaultCategoryTrace);
            WriteLine("Framework: " + Environment.Version.ToString(), defaultCategoryTrace);
            WriteLine("Anwender:  " + Environment.UserName, defaultCategoryTrace);
            WriteLine("Domain:    " + Environment.UserDomainName, defaultCategoryTrace);
            WriteLine("Hostname:  " + Environment.MachineName, defaultCategoryTrace);
            WriteLine("-------------------------------------------------------------------------", defaultCategoryTrace);
            WriteLine("Thread:    " + Thread.CurrentThread.ManagedThreadId.ToString(), defaultCategoryTrace);
            WriteLine("Process:   " + process.Id.ToString(), defaultCategoryTrace);
            WriteLine("Session:   " + process.SessionId.ToString(), defaultCategoryTrace);
            WriteLine("-------------------------------------------------------------------------", defaultCategoryTrace);
            WriteLine("", defaultCategoryTrace);

            if (!isOpen)
                fileClose();
        }

        /// <summary>
        /// Schreibt Informationen zum Dateiwechsel in die Ausgabedatei
        /// </summary>
        /// <param name="switchReason"></param>
        private void writeSwitchReason(string switchReason)
        {
#if DEBUG
            WriteLine(switchReason + " - DebugVersion", defaultCategoryTrace);
#else
            WriteLine(switchReason + " - ReleaseVersion", defaultCategoryTrace);
#endif
            WriteLine("In der aktuellen Session wurden bisher " + loggedMessages.ToString() + " Nachrichten geschrieben", defaultCategoryTrace);
        }

        /// <summary>
        /// Schreibt eine Nachricht und zusätzliche Ereignisinformationen mit Kategorie in die Ausgabedatei.
        /// </summary>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        /// <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        /// <param name="dateTime">Datum und Uhrzeit, an dem das Ereignis aufgetreten ist.</param>
        /// <param name="threadId">Eindeutiger Bezeichner für den aktuellen verwalteten Thread.</param>
        /// <param name="processId">Eindeutiger Bezeichner des aktuellen Prozesses.</param>
        private void writeLine(string message, string category, long id, DateTime dateTime, int threadId, int processId)
        {
            fileOpen();

            checkSwitchReason();

            if (category.Length > maxCategoryNameLength)
                category = category.Substring(0, maxCategoryNameLength);

            string formatString;
            if (outputFormat == OutputFormat.CSV || outputFormat == OutputFormat.ExcelCSV)
            {
                if (outputFormat == OutputFormat.ExcelCSV)
                    formatString = messageFormatStringCSV.Replace(",", ";");
                else
                    formatString = messageFormatStringCSV;

                category = string.Format("\"{0}\"", category);
                if (message.IndexOfAny(new char[] { '"', ',', ';' }) != -1)
                    message = message.Replace("\"", "\"\"");
                message = string.Format("\"{0}\"", message);
            }
            else
                formatString = messageFormatStringFixedFirstLine;

            if (outputFormat == OutputFormat.CSV || outputFormat == OutputFormat.ExcelCSV)
            {
                logFileStream.WriteLine(formatString,
                    category.ToUpper(),
                    id,
                    threadId,
                    processId,
                    dateTime.ToString("yyyy-MM-dd HH:mm:ss"),
                    message);
            }
            else
            {
                string[] lines = message.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                bool firstLine = true;

                foreach (string line in lines)
                {
                    if (firstLine)
                    {
                        logFileStream.WriteLine(formatString,
                            category.ToUpper(),
                            id,
                            threadId,
                            processId,
                            dateTime.ToString("yyyy-MM-dd HH:mm:ss"),
                            line);
                        firstLine = false;
                    }
                    else
                        logFileStream.WriteLine(messageFormatStringFixedLines, line);
                }
            }

            loggedMessages++;

            fileClose();
        }

        #endregion

        #region Public override Methoden

        #region Write-Methode

        /// <summary>
        /// Schreibt eine Nachricht in die Ausgabedatei.
        /// </summary>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        /// <remarks>Funktioniert analog zur <see cref="WriteLine(string message)"/> Methode. Ruft diese direkt auf.</remarks>
        public override void Write(string message)
        {
            this.WriteLine(message);
        }

        /// <summary>
        /// Schreibt eine Nachricht mit Kategorie in die Ausgabedatei.
        /// </summary>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        /// <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        /// <remarks>Funktioniert analog zur <see cref="WriteLine(string message, string category)"/> Methode. Ruft diese direkt auf.</remarks>
        public override void Write(string message, string category)
        {
            this.WriteLine(message, category);
        }

        /// <summary>
        /// Schreibt den Wert der ToString-Methode eines Objekts in die Ausgabedatei.
        /// </summary>
        /// <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        /// <remarks>Funktioniert analog zur <see cref="WriteLine(object o)"/> Methode. Ruft diese direkt auf.</remarks>
        public override void Write(object o)
        {
            this.WriteLine(o);
        }

        /// <summary>
        /// Schreibt den Wert der ToString-Methode eines Objekts mit Kategorie in die Ausgabedatei.
        /// </summary>
        /// <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        /// <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        /// <remarks>Funktioniert analog zur <see cref="WriteLine(object o, string category)"/> Methode. Ruft diese direkt auf.</remarks>
        public override void Write(object o, string category)
        {
            this.WriteLine(o, category);
        }

        #endregion

        #region WriteLine-Methode

        /// <summary>
        /// Schreibt eine Nachricht in die Ausgabedatei.
        /// </summary>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        public override void WriteLine(string message)
        {
            this.WriteLine(message, defaultCategory);
        }

        /// <summary>
        /// Schreibt eine Nachricht mit Kategorie in die Ausgabedatei.
        /// </summary>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        /// <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        public override void WriteLine(string message, string category)
        {
            int threadId = Thread.CurrentThread.ManagedThreadId;
            int processId = Process.GetCurrentProcess().Id;

            this.writeLine(message, category, 0, DateTime.Now, threadId, processId);
        }

        /// <summary>
        /// Schreibt den Wert der ToString-Methode eines Objekts in die Ausgabedatei.
        /// </summary>
        /// <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        public override void WriteLine(object o)
        {
            this.WriteLine(o, defaultCategory);
        }

        /// <summary>
        /// Schreibt den Wert der ToString-Methode eines Objekts mit Kategorie in die Ausgabedatei.
        /// </summary>
        /// <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        /// <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        public override void WriteLine(object o, string category)
        {
            this.WriteLine(o.ToString(), category);
        }

        #endregion

        #region TraceData-Methode

        /// <summary>
        /// Schreibt Ablaufverfolgungsinformationen, ein Datenobjekt und Ereignisinformationen in die spezifische Ausgabe des Listeners.
        /// </summary>
        /// <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        /// <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        /// <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        /// <param name="data">Die auszugebenden Ablaufverfolgungsdaten.</param>
        /// <remarks>Informationen </remarks>
        public override void TraceData(TraceEventCache eventCache, string source, TraceEventType eventType, int id, object data)
        {
            this.writeLine(source + ": " + data.ToString(), eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId);
        }

        /// <summary>
        /// Schreibt Ablaufverfolgungsinformationen, ein Array von Datenobjekten und Ereignisinformationen in die spezifische Ausgabe des Listeners.
        /// </summary>
        /// <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        /// <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        /// <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        /// <param name="data">Ein Objektarray, das in Form von Daten ausgegeben werden soll.</param>
        public override void TraceData(TraceEventCache eventCache, string source, TraceEventType eventType, int id, params object[] data)
        {
            string message = "";

            foreach (object obj in data)
            {
                if (message != "")
                    message += ", ";
                message += obj.ToString();
            }
            this.writeLine(source + ": " + message, eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId);
        }

        #endregion

        #region TraceEvent-Methode

        /// <summary>
        /// Schreibt Ablaufverfolgungs- und Ereignisinformationen in die Ausgabedatei.
        /// </summary>
        /// <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        /// <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        /// <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id)
        {
            this.TraceEvent(eventCache, source, eventType, id, "");
        }

        /// <summary>
        /// Schreibt Ablaufverfolgungsinformationen, eine Meldung und Ereignisinformationen in die Ausgabedatei. 
        /// </summary>
        /// <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        /// <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        /// <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id, string message)
        {
            if ((eventType == TraceEventType.Error || eventType == TraceEventType.Critical) && eventCache.Callstack != "")
            {
                if (message != "")
                    message += Environment.NewLine;
                message += eventCache.Callstack;
            }
            this.writeLine(source + ": " + message, eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId);
        }

        /// <summary>
        /// Schreibt Ablaufverfolgungsinformationen, ein formatiertes Array von Objekten und Ereignisinformationen in die Ausgabedatei. 
        /// </summary>
        /// <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        /// <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        /// <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        /// <param name="format">Eine Formatzeichenfolge mit 0 oder mehr Formatelementen, die Objekten im args-Array entsprechen.</param>
        /// <param name="args">Ein object-Array mit 0 oder mehr zu formatierenden Objekten.</param>
        public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id, string format, params object[] args)
        {
            if (args != null)
                this.TraceEvent(eventCache, source, eventType, id, string.Format(format, args));
            else
                this.TraceEvent(eventCache, source, eventType, id, format);
        }

        #endregion

        #region TraceTransfer-Methode

        /// <summary>
        /// Schreibt Ablaufverfolgungsinformationen, eine Meldung, eine verwandte Aktivitätsidentität und Ereignisinformationen in die Ausgabedatei.
        /// </summary>
        /// <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        /// <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        /// <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        /// <param name="message">Eine zu schreibende Meldung.</param>
        /// <param name="relatedActivityId">Ein Guid-Objekt, das eine verwandte Aktivität bezeichnet.</param>
        public override void TraceTransfer(TraceEventCache eventCache, string source, int id, string message, Guid relatedActivityId)
        {
            TraceEventType eventType = TraceEventType.Transfer;
            this.writeLine(source + ": " + relatedActivityId.ToString() + ": " + message, eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId);
        }

        #endregion

        #endregion
    }
}